/* This file is a part of the program Kira.
 * Copyright (C) Johann Usovitsch <jusovitsch@googlemail.com>
 * Philipp Maierhoefer <particle@maierhoefer.net>
 * Peter Uwer <peter.uwer@physik.hu-berlin.de>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version , or (at
 * your option) any later version as published by the Free Software 
 * Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
*/
#ifndef COEFF_INT_H
#define COEFF_INT_H

#include <iostream>
#include <string>
#include <vector>
#include <utility>

namespace pyred
{

class Coeff_int
{
  friend Coeff_int operator-(const Coeff_int&);
  friend Coeff_int operator+(const Coeff_int&, const Coeff_int&);
  friend Coeff_int operator-(const Coeff_int&, const Coeff_int&);
  friend Coeff_int operator*(const Coeff_int&, const Coeff_int&);
  friend Coeff_int operator/(const Coeff_int&, const Coeff_int&);
  friend bool operator==(const Coeff_int&, const Coeff_int&);
  friend std::ostream& operator<<(std::ostream&, const Coeff_int&);
private:
  uint64_t c;
  static uint64_t prime;
  static bool auto_vars;
  static std::string set_to_one;
  static std::vector<std::pair<std::string,uint64_t>> randvars;
public:
  static void init(const std::vector<std::string>& = {}, uint = 1,
                   const std::string& = "");
  Coeff_int();
  Coeff_int(const Coeff_int&);
  Coeff_int(const std::string&);
  template<typename T> // only for integral types
  Coeff_int(T);
  Coeff_int& operator+=(const Coeff_int&);
  Coeff_int& operator-=(const Coeff_int&);
  Coeff_int& operator*=(const Coeff_int&);
  Coeff_int& operator/=(const Coeff_int&);
  Coeff_int& operator^=(const Coeff_int&);
};

Coeff_int operator-(const Coeff_int&);
Coeff_int operator+(const Coeff_int&, const Coeff_int&);
Coeff_int operator-(const Coeff_int&, const Coeff_int&);
Coeff_int operator*(const Coeff_int&, const Coeff_int&);
Coeff_int operator/(const Coeff_int&, const Coeff_int&);
bool operator==(const Coeff_int&, const Coeff_int&);
std::ostream& operator<<(std::ostream&, const Coeff_int&);

template<typename T>
inline Coeff_int::Coeff_int(T a) {
  if (a >= 0) {
    c = uint64_t(a);
  }
  else {
    c = prime - uint64_t(-a);
  }
}

}

#endif
